# -*- coding: utf-8 -*-
"""
Created on Tue Mar  7 10:15:55 2023

@author: pawelzik
"""

import pandas as pd
import sys
from pathlib import Path
import os
import MeasurementPlot
import curvefit_and_correlation


# after measurement has finished from stored csv-data a post plot of all temperature steps is plotted and store in
# results folder. class generates a result plot, Michael
# oftional correclation and regression can be calculated and plotted in results, Michael
# option is configureable via ext_sens_data.json, Michael
class PostPlot:
    def __init__(self):
          # set python for opening an separate plot window
        if 'ipykernel' in sys.modules:
            from IPython import get_ipython
            get_ipython().run_line_magic('matplotlib', 'qt')
        
        
        self.measplot_obj = MeasurementPlot.MeasurementPlot(legend_loc = 'upper left', \
                                                       legend_bbox_to_anchor = (1.09, 1))
        
            # set parameter figure of class to object parameter figure
        self.fig = self.measplot_obj.fig
    
    # read csv-file and import data to data frame
    def import_csv(self, csv_file):
        
        data_frame = pd.read_csv(csv_file)
        
        return data_frame
                
    # plot function for data frame data, Michael
    def plot_frame_data(self, data_frame, title ='', time_unit ='min'):
        
        # set title of plot 
        self.fig.suptitle("Measurement "+title, color="red")
        
        # reset index of data frame
        data_frame.reset_index(inplace=True, drop=True)
        
        # set y-Achlabel in all subplots to Time an in square brackets the selected time unit 
        for element in self.measplot_obj.ax1:
            element.set_xlabel("Time [%s]" %time_unit)         
        
        # make a copy of data_frame in parameterlist without changing original during modification
        postplot_data_frame = data_frame.copy()
        
        # time stamp of index = 0 is the start point of time axis, Michael
        # substract all other timestamps with time stamp of index zero to get time scale in
        # seconds, Michael
        time_sec = postplot_data_frame.TIMESTAMP - postplot_data_frame.TIMESTAMP[0]
        
        # set scaling of time axis depending on the time unit given in parameter list, Michael
        # default unit is minutes, Michael
        time_vals = self.scaling_time_axes(time_sec, time_unit)
        
        # update Timestamps with calculated time values         
        postplot_data_frame.update(time_vals) 
        
        # refresh subflots with data in data frame
        self.measplot_obj.draw(postplot_data_frame, pdf_name = '')
        
        # cal PK2PK values of magnitude and phase 
        PK2PK = self.calc_mag_phase_pkpk_values(data_frame)

        self.edit_annotation_in_plot(annotate_string  = PK2PK)
        
        
    
    def edit_annotation_in_plot(self, annotate_string  ='', anno_fontsize = 16):
        
        self.fig
        
        # update text of annotation in first subplot
        self.measplot_obj.annotation.set_text(annotate_string)
        
        # edit annotation fontsize in first subplot 
        self.measplot_obj.annotation.set_fontsize(anno_fontsize)
    
    def calc_mag_phase_pkpk_values(self, data_frame):
        
        # calc PK2PK values of magnitude and phase
        delta_phase = max(data_frame.S21_PHASE) - min(data_frame.S21_PHASE)
        delta_magnitude = max(data_frame.S21_MAGNITUDE) - min(data_frame.S21_MAGNITUDE)
        
        # generate text for annoation in first subplot 
        delta_vals_string = "$\Delta\phi$($S_{21PkPk}$): %.3f °\n$\Delta$|$S_{21PkPk}$|: %.3f dB"  \
            % (delta_phase, delta_magnitude)
            
        return delta_vals_string
       
 
    # save figure under the given storepath in parameterlist with the given filename in parameter list, Michael
    # file extension is part of filename, Michael
    def save_fig(self, storepath, filename):
        if not os.path.exists(storepath):
            os.makedirs(storepath)
        
        self.fig.savefig(storepath + '\\' + filename)
    
    # scaling time axes depending on the time unit
    def scaling_time_axes(self, time_sec, time_unit):
        if time_unit == 'min':
            time_vals = time_sec/60
        
        elif time_unit == 'hours':
            time_vals = time_sec/3600
        
        elif time_unit == 'sec':
            time_vals = time_sec
            
        else:
            time_vals = time_sec/60
    
        return time_vals
    
       

    
# for manually redo post plot after measurement has finished, Michael
if __name__ == '__main__':
    
    
    # set result path for post plot 
    Results_Path = r'C:\Users\pawelzik\Desktop\Results\JBY240_14032023Copy'
    
    
    # set time unit for post post plot
    # default is minutes if entry in parameterlist left empty
    # possible entries: 'min' for minutes, 'hours' for hours and 'sec' for seconds
    time_unit = 'min'
    
    # set storepath for the post plots, Michael
    storepath = Results_Path + '\\Plots' 
    
    # search all csv files in results folder 
    csv_file_list = list(Path(Results_Path).glob("**/*.csv"))
    
    # choose annotation option
    plot_correlation_coeff = False
    plot_regression_coeff = False
    
    # activate plot curvefitting
    
    plot_trace_curvefit = False 
    
    
    # create postplot object, Michael
    plot_obj = PostPlot()
    
    fit_corr_obj = curvefit_and_correlation.reg_and_corr(plot_obj)
    
    # empty data frame for concat the data frames from csv import to plot full transistion
    concat_data_frame = pd.DataFrame()
    
    # plot results for each csv-file
    for index, csv_file in enumerate(csv_file_list):
    
        # import csv-data from csv-files in list, Michael
        data_frame = plot_obj.import_csv(str(csv_file))
        
        # determine title of plot from csv-filename, Michael
        title = csv_file.name
        
        # concate datesframe for plotting full transistion data 
        concat_data_frame = pd.concat([concat_data_frame,data_frame],ignore_index=True, sort = False)       
        
        
        # plot frame data
        plot_obj.plot_frame_data(data_frame, title, time_unit)
        
        
        if plot_correlation_coeff == True:
            
            # determine correlation coefficient between func 1 and func 2
            corr_coeff =fit_corr_obj.calc_correlation(func_1 = data_frame.S21_PHASE , \
                                      func_2 = data_frame.TEMP_DUT)
            
            # plot correlation coefficient
            plot_obj.edit_annotation_in_plot(annotate_string  = corr_coeff)
        
        elif plot_regression_coeff == True:
            
            # start curvefit and determine coefficients best fit function
            reg_coeff = fit_corr_obj.calc_regression_coeff_phase_S21(data_frame, time_unit = 'min')
            
            # plot regression coefficients
            plot_obj.edit_annotation_in_plot(annotate_string  = reg_coeff)
        
            plot_obj.edit_annotation_in_plot(annotate_string  = reg_coeff)
        
        if plot_trace_curvefit == True:
            
            fit_corr_obj.plot_phase_curve_fit(data_frame = data_frame, \
                                          postplot_obj = plot_obj.measplot_obj, \
                                              time_unit = time_unit, state = True)
        
        # set filename of post plot, Michael
        filename = str(csv_file.stem) + '.pdf'
        
        # store post plot under the path taht is set in storepath with the earlier defined filename
        plot_obj.save_fig(storepath, filename)
    
      
    # plot of all steps of a sweep is plotted in one diagram, Michael
    # title of this plot will be always full transistion, Michel
    plot_obj.plot_frame_data(concat_data_frame, 'Full Transistion', time_unit)
    
    fit_corr_obj.plot_phase_curve_fit(data_frame = concat_data_frame, \
                                          postplot_obj = plot_obj.measplot_obj, \
                                              time_unit = time_unit, state = False)
    
    # filename of plot that contains all steps of sweep is set to FullTransistion, Michael
    filename = 'Full_Transistion' + '.pdf'
    
    # plot with the results of all steps is store under the predefined storpath with the earlier defined filename, Michael
    plot_obj.save_fig(storepath, filename)

  